/*******************************************************************************
* Copyright (C) 2019 China Micro Semiconductor Limited Company. All Rights Reserved.
*
* This software is owned and published by:
* CMS LLC, No 2609-10, Taurus Plaza, TaoyuanRoad, NanshanDistrict, Shenzhen, China.
*
* BY DOWNLOADING, INSTALLING OR USING THIS SOFTWARE, YOU AGREE TO BE BOUND
* BY ALL THE TERMS AND CONDITIONS OF THIS AGREEMENT.
*
* This software contains source code for use with CMS
* components. This software is licensed by CMS to be adapted only
* for use in systems utilizing CMS components. CMS shall not be
* responsible for misuse or illegal use of this software for devices not
* supported herein. CMS is providing this software "AS IS" and will
* not be responsible for issues arising from incorrect user implementation
* of the software.
*
* This software may be replicated in part or whole for the licensed use,
* with the restriction that this Disclaimer and Copyright notice must be
* included with each copy of this software, whether used in part or whole,
* at all times.
*/

/****************************************************************************/
/** \file FLASH.c
**
** 
**
**	History:
**	
*****************************************************************************/
/****************************************************************************/
/*	include files
*****************************************************************************/
#include "flash.h"

/****************************************************************************/
/*	Local pre-processor symbols/macros('#define')
****************************************************************************/


/****************************************************************************/
/*	Global variable definitions(declared in header file with 'extern')
*****************************************************************************/

/****************************************************************************/
/*	Local type definitions('typedef')
*****************************************************************************/

/****************************************************************************/
/*	Local variable  definitions('static')
*****************************************************************************/

/****************************************************************************/
/*	Local function prototypes('static')
*****************************************************************************/

/****************************************************************************/
/*	Function implementation - global ('extern') and local('static')
*****************************************************************************/
/*****************************************************************************
 ** \brief	FLASH_UnLock
 **			
 ** \param [in] none
 ** \return  none
 ** \note	 
*****************************************************************************/
void FLASH_UnLock(void)
{
	if (EA == 1) //TAʱʱϣҪرж
	{
		EA = 0;
		TA = 0xAA;
		TA = 0x55;
		MLOCK = 0xAA;
		EA = 1;
	}
	else
	{
		TA = 0xAA;
		TA = 0x55;
		MLOCK = 0xAA;
	}
}
/*****************************************************************************
 ** \brief	FLASH_Lock
 **			
 ** \param [in] none
 ** \return  none
 ** \note	 
*****************************************************************************/
void FLASH_Lock(void)
{
	if (EA == 1) //TAʱʱϣҪرж
	{
		EA = 0;
		TA = 0xAA;
		TA = 0x55;
		MLOCK = 0x00;
		EA = 1;
	}
	else
	{
		TA = 0xAA;
		TA = 0x55;
		MLOCK = 0x00;
	}
}


/*****************************************************************************
 ** \brief	FLASH_ReadMLockFlag
 **			ȡMLCOK־λ
 ** \param [in] none
 ** \return  0ɲFLASH  1ɲFLASH
 ** \note	 
*****************************************************************************/
uint8_t  FLASH_ReadMLockFlag(void)
{
	return ((MSTATUS & FLASH_MSTATUS_MLOCK_Msk)? 1:0);
}

/*****************************************************************************
 ** \brief	FLASH_ReadErrorFlag
 **			ȡ־λ
 ** \param [in] none
 ** \return  0޴  1
 ** \note	 
*****************************************************************************/
uint8_t  FLASH_ReadErrorFlag(void)
{
	return ((MSTATUS & FLASH_MSTATUS_ERROR_Msk)? 1:0);
}

/*****************************************************************************
 ** \brief	FLASH_Write
 **			д
 ** \param [in] FLASHModule	(1)FLASH_DATA: DATA
 **							  (2)FLASH_APROMAPROM
 **							  (3)FLASH_BOOT : BOOT
 **				Addr: 18bit	(1)DATA 0x00~0xFFF
 **							  (2)APROM0x00~0xFFFF
 **							  (3)BOOT0x00~0x3FFF  
 **				Data: 8bit
 ** \return  none
 ** \note	 
*****************************************************************************/
void FLASH_Write(uint8_t FLASHModule, uint32_t Addr, uint8_t Data)
{
	uint8_t temp;
	
	MREGION = FLASHModule;
	
	temp = MSTATUS;
	temp &= ~(FLASH_MSTATUS_MADR_Msk);
	temp |= 0x3 & (Addr >>16);
	MADRH = 0xFF & (Addr>>8);
	MADRL = 0xFF & Addr;	
	
	MDATA = Data;	
	
	MMODE = FLASH_WRITE;
	if(EA == 1)
	{
		EA = 0;
		MSTATUS = temp | FLASH_MSTATUS_START_Msk;//FLASHڼ䲻
		_nop_();
		_nop_();
		_nop_();
		_nop_();
		_nop_();
		_nop_();
		while(MSTATUS & 0x08);
		EA = 1;
	}
	else
	{
		MSTATUS = temp | FLASH_MSTATUS_START_Msk;
		_nop_();
		_nop_();
		_nop_();
		_nop_();
		_nop_();
		_nop_();
		while(MSTATUS & 0x08);		
	}
}

/*****************************************************************************
 ** \brief	FLASH_Read
 **			
 ** \param [in] FLASHModule	(1)FLASH_DATA: DATA
 **							  (2)FLASH_APROMAPROM
 **							  (3)FLASH_BOOT : BOOT
 **				Addr: 18bit	(1)DATA 0x00~0xFFF
 **							  (2)APROM0x00~0xFFFF
 **							  (3)BOOT0x00~0x3FFF  
 ** \return  8bit Data
 ** \note	
*****************************************************************************/
uint8_t FLASH_Read(uint8_t FLASHModule, uint32_t Addr)
{
	uint8_t temp;
	
	MREGION = FLASHModule;
	
	temp = MSTATUS;
	temp &= ~(FLASH_MSTATUS_MADR_Msk);
	temp |= 0x3 & (Addr >>16);
	MADRH = 0xFF & (Addr>>8);
	MADRL = 0xFF & Addr;	
		
	MMODE = FLASH_READ;
	
	if(EA == 1)
	{
		EA = 0;
		MSTATUS = temp | FLASH_MSTATUS_START_Msk;//FLASHڼ䲻

		_nop_();
		_nop_();
		_nop_();
		_nop_();
		_nop_();
		_nop_();
		while(MSTATUS & 0x08);
		EA = 1;
	}
	else
	{
		MSTATUS = temp | FLASH_MSTATUS_START_Msk;

		_nop_();
		_nop_();
		_nop_();
		_nop_();
		_nop_();
		_nop_();	
		while(MSTATUS & 0x08);
	}
	
	return (MDATA);
}

/*****************************************************************************
 ** \brief	FLASH_Erase
 **			
 ** \param [in] FLASHModule	(1)FLASH_DATA: DATA
 **							  (2)FLASH_APROMAPROM
 **							  (3)FLASH_BOOT : BOOT
 **				Addr: 18bit	(1)DATA 0x00~0xFFF
 **							  (2)APROM0x00~0xFFFF
 **							  (3)BOOT0x00~0x3FFF  
 ** \return  none
 ** \note	
*****************************************************************************/
void FLASH_Erase(uint8_t FLASHModule, uint32_t Addr)
{
	uint8_t temp;
	
	MREGION = FLASHModule;
	
	temp = MSTATUS;
	temp &= ~(FLASH_MSTATUS_MADR_Msk);
	temp |= 0x3 & (Addr >>16);
	MADRH = 0xFF & (Addr>>8);
	MADRL = 0xFF & Addr;	
		
	MMODE = FLASH_ERASE;
	
	if(EA == 1)
	{
		EA = 0;
		MSTATUS = temp | FLASH_MSTATUS_START_Msk;//FLASHڼ䲻

		_nop_();
		_nop_();
		_nop_();
		_nop_();
		_nop_();
		_nop_();
		while(MSTATUS & 0x08);
		EA = 1;
	}
	else
	{
		MSTATUS = temp | FLASH_MSTATUS_START_Msk;

		_nop_();
		_nop_();
		_nop_();
		_nop_();
		_nop_();
		_nop_();
		while(MSTATUS & 0x08);
	}
}

/*****************************************************************************
 ** \brief	FLASH_Crc
 **			CRCУ
 ** \param [in] FLASHModule	(1)FLASH_DATA: DATA
 **							  (2)FLASH_APROMAPROM
 **							  (3)FLASH_BOOT : BOOT
 **				SAddr/EAddr: 18bit(1)DATA 0x00~0xFFF
**						  			(2)APROM0x00~0xFFFF
 **							 	    (3)BOOT0x00~0x3FFF  
 ** \return 16λУֵ
 ** \note	
*****************************************************************************/
uint16_t  FLASH_Crc(uint8_t FLASHModule, uint32_t SAddr, uint32_t EAddr)
{
	uint16_t crc;
	if(EA == 1)    //FLASHڼ䲻
	{
		EA = 0;
	
		PCRCDL =0x00;
		PCRCDH =0x00;
		
		MSTATUS &= ~(FLASH_MSTATUS_MADR_Msk | FLASH_MSTATUS_CRCASEL_Msk);
		MSTATUS |= 0x3 & (SAddr >>16);
		MADRH = 0xFF & (SAddr>>8);
		MADRL = 0xFF & SAddr;	
		
		MSTATUS |= FLASH_MSTATUS_CRCASEL_Msk;	
		MSTATUS &= ~(FLASH_MSTATUS_MADR_Msk);
		MSTATUS |= (0x3 & (EAddr >>16));
		MADRH = 0xFF & (EAddr>>8);
		MADRL = 0xFF & EAddr;		
		
		MREGION = FLASHModule;
		
		MMODE = FLASH_CRC;
		MSTATUS |= FLASH_MSTATUS_START_Msk;	

		_nop_();
		_nop_();
		_nop_();
		_nop_();
		_nop_();
		_nop_();
		
		if(MSTATUS & FLASH_MSTATUS_START_Msk)
			crc =0;
		else	
			crc = PCRCDL | (PCRCDH<<8);
		EA = 1;
	}
	else
	{
		PCRCDL =0x00;
		PCRCDH =0x00;
		
		MSTATUS &= ~(FLASH_MSTATUS_MADR_Msk | FLASH_MSTATUS_CRCASEL_Msk);
		MSTATUS |= 0x3 & (SAddr >>16);
		MADRH = 0xFF & (SAddr>>8);
		MADRL = 0xFF & SAddr;	
		
		MSTATUS |= FLASH_MSTATUS_CRCASEL_Msk;	
		MSTATUS &= ~(FLASH_MSTATUS_MADR_Msk);
		MSTATUS |= (0x3 & (EAddr >>16));
		MADRH = 0xFF & (EAddr>>8);
		MADRL = 0xFF & EAddr;		
		
		MREGION = FLASHModule;
		
		MMODE = FLASH_CRC;
		MSTATUS |= FLASH_MSTATUS_START_Msk;	

		_nop_();
		_nop_();
		_nop_();
		_nop_();
		_nop_();
		_nop_();
		
		if(MSTATUS & FLASH_MSTATUS_START_Msk)
			crc =0;
		else	
			crc = PCRCDL | (PCRCDH<<8);
	}
	return crc;
}